package hero.util;

/**
*
* Bonita
* Copyright (C) 1999 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
*
--------------------------------------------------------------------------
* $Id: BonitaConfigService.java,v 1.2 2004/11/29 12:29:48 mvaldes Exp $
*
--------------------------------------------------------------------------
*/

import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectConfigLocal;

import java.util.Collection;
import java.util.Iterator;

import javax.ejb.FinderException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.NotCompliantMBeanException;
import javax.management.MalformedObjectNameException;
import javax.naming.Context;
import org.objectweb.jonas.service.ServiceException;

//import objectweb
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceManager;

public class BonitaConfigService extends AbsServiceImpl implements BonitaConfigServiceMBean{

	private MBeanServer mbeanServer = null;
	public final static String SERVICE_NAME = "jonas:type=service,name=BonitaConfigService";

	public void doInit(Context ctx) throws ServiceException {
		// Get the JMX Server via JMX Service
          try {
	           JmxService jmx = (JmxService) ServiceManager.getInstance().getJmxService();
			   mbeanServer = jmx.getJmxServer();		     
		   } catch (Exception e) {
			//the JMX service may not be started
			throw new ServiceException("JMX Server is not started...");
		   } 			 	      
	}
	
	public void doStart() throws ServiceException {
		try {
			mbeanServer.registerMBean(this,new ObjectName(SERVICE_NAME));
 	        System.out.println("BonitaConfigService :: Start Service");
		} catch (InstanceAlreadyExistsException iae) {
			throw new ServiceException("Cannot start the BonitaConfigService (Already Exists)" , iae);
		} catch (MBeanRegistrationException mbre) {
			throw new ServiceException("Cannot start the BonitaConfigService (MBean registration error)" , mbre);
		} catch (NotCompliantMBeanException ncmbe) {
			throw new ServiceException("Cannot start the BonitaConfigService (MBean Not compliant error)" , ncmbe);
		} catch (MalformedObjectNameException mone) {
			throw new ServiceException("Cannot start the BonitaConfigService (ObjectName Malformed)" , mone);
		}
	}
	
	public void doStop() throws ServiceException {
		if (mbeanServer != null) {
			try {
				mbeanServer.unregisterMBean(new ObjectName(SERVICE_NAME));
			} catch (Exception e) {
				throw new ServiceException("Cannot stop the BonitaConfigService (JMX).", e);
			}
		}
		System.out.println("BonitaConfigService :: Stop Service");
	}
	
	public boolean getProcessJMS(String projectName) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
	 	return(pc.getJMS());		
	}
	
	public boolean getProcessLog(String projectName) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		return(pc.getLog());		 	 		 	 		
		
	}
	
	public boolean getProcessTrace(String projectName) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		return(pc.getTrace());		 	 		 	 				
	}
	
	public String getProcessLogLevel(String projectName) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		return(pc.getLogLevel());		 	 		 	 		
	}
	
	public String getProcessTraceLevel(String projectName) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		return(pc.getTraceLevel());		 	 		 	 				
	}
	
	public String getProcessHistoric(String projectName) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		return(pc.getHistoric());		 	 		 	 				
	}
	
	public void setProcessJMS(String projectName, boolean value) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		pc.setJMS(value);
	}
	
	public void setProcessLog(String projectName, boolean value) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		pc.setLog(value);		
	}
	
	public void setProcessTrace(String projectName, boolean value) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
	 	pc.setTrace(value);		
	}
	
	public void setProcessTraceLevel(String projectName, String value) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		pc.setTraceLevel(value);		
	}
	
	public void setProcessLogLevel(String projectName, String value) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		pc.setLogLevel(value);		
	}
	
	public void setProcessHistoric(String projectName, String value) throws ServiceException {
		BnProjectConfigLocal pc = this.getProjectConfig(projectName);
		pc.setHistoric(value);	
	}
	
	public void updateJMS(boolean value) throws ServiceException {
		Collection projects = this.getProjects();
		Iterator prjs = projects.iterator();
		while (prjs.hasNext()) {
			BnProjectLocal pj = (BnProjectLocal) prjs.next();
			BnProjectConfigLocal pc = pj.getBnProjectConfig();
			pc.setJMS(value);
		}
	}

	public void updateLog(boolean value) throws ServiceException {
		Collection projects = this.getProjects();
		Iterator prjs = projects.iterator();
		while (prjs.hasNext()) {
			BnProjectLocal pj = (BnProjectLocal) prjs.next();
			BnProjectConfigLocal pc = pj.getBnProjectConfig();
			pc.setLog(value);
		}

	}

	public void updateTrace(boolean value) throws ServiceException {
		Collection projects = this.getProjects();
		Iterator prjs = projects.iterator();
		while (prjs.hasNext()) {
			BnProjectLocal pj = (BnProjectLocal) prjs.next();
			BnProjectConfigLocal pc = pj.getBnProjectConfig();
			pc.setTrace(value);
		}

	}

	public void updateLogLevel(String value) throws ServiceException {
		Collection projects = this.getProjects();
		Iterator prjs = projects.iterator();
		while (prjs.hasNext()) {
			BnProjectLocal pj = (BnProjectLocal) prjs.next();
			BnProjectConfigLocal pc = pj.getBnProjectConfig();
			pc.setLogLevel(value);
		}
	}

	public void updateTraceLevel(String value) throws ServiceException {
		Collection projects = this.getProjects();
		Iterator prjs = projects.iterator();
		while (prjs.hasNext()) {
			BnProjectLocal pj = (BnProjectLocal) prjs.next();
			BnProjectConfigLocal pc = pj.getBnProjectConfig();
			pc.setTraceLevel(value);
		}
	}

	public void updateHistoric(String value) throws ServiceException {
		Collection projects = this.getProjects();
		Iterator prjs = projects.iterator();
		while (prjs.hasNext()) {
			BnProjectLocal pj = (BnProjectLocal) prjs.next();
			BnProjectConfigLocal pc = pj.getBnProjectConfig();
			pc.setHistoric(value);
		}
	}
		
	private BnProjectConfigLocal getProjectConfig(String projectName) throws ServiceException{
		BnProjectLocalHome phome = null;
		BnProjectLocal plocal = null;
		try {
		 	 phome = hero.interfaces.BnProjectUtil.getLocalHome();
		 	 plocal = phome.findByName(projectName);
		 	 return(plocal.getBnProjectConfig());
		 } catch (javax.naming.NamingException ne) {
			 throw new ServiceException(ne.getMessage());
		 } catch (FinderException fie) {
			 throw new ServiceException(fie.getMessage());
		 }
	}
	
	private Collection getProjects() throws ServiceException{	
		BnProjectLocalHome phome = null;
		BnProjectLocal plocal = null;
		try {
			phome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection projects = phome.findAll();
			return projects;		 	 
		} catch (javax.naming.NamingException ne) {
			throw new ServiceException(ne.getMessage());
		} catch (FinderException fie) {
			throw new ServiceException(fie.getMessage());
		}	
	}
}
