/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BonitaServiceLocator.java,v 1.3 2004/10/13 13:06:22 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.util;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Map;
import java.util.HashMap;
import javax.naming.ServiceUnavailableException;
import hero.interfaces.SequenceGenerator;
import hero.interfaces.SequenceGeneratorHome;

import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;
import javax.mail.Session;

public class BonitaServiceLocator {
    private static BonitaServiceLocator msl;
    private Map cache;
    private static Context initial;

   
    private BonitaServiceLocator() throws BonitaServiceException {
	try {
	    initial=new InitialContext();
	    cache = new HashMap();
	} catch (javax.naming.NamingException ne) {
	    throw new BonitaServiceException(ne.getMessage());
	}
    }

    static public BonitaServiceLocator getInstance() throws BonitaServiceException {

	if (msl==null) {
	    msl=new BonitaServiceLocator();
	}
	return msl;
    }

    // Services constant inner class - service objects

    public class Services {
	   final public static int TOPIC_CONNECTION_FACTORY=0;
	   final public static int TOPIC=1;
	   final public static int MAIL_SERVICE=2;
    }

    // JMS related constants
    
    final static String TOPIC_CONNECTION_FACTORY_NAME = "TCF";
	final static String TOPIC_NAME = "testTopic";

    // Mail configuration
    final static String MAIL_SERVICE_NAME = "Mail";    

    // Returns the JNDI name for the required service
    
	// Entity beans sequence names
    
 	final public static String AGENT = "BnAgent";
	final public static String AGENT_EDGE = "BnAgentEdge";
	final public static String AUTH_ROLE = "BnAuthRole";
	final public static String EDGE = "BnEdge";
	final public static String NODE = "BnNode";
	final public static String NODE_HOOK = "BnNodeHook";
	final public static String NODE_INTER_HOOK = "BnNodeInterHook";
	final public static String NODE_PERFORMER_ASSIGN = "BnNodePerformerAssign";
	final public static String NODE_PROPERTY = "BnNodeProperty";
	final public static String PROJECT = "BnProject";
	final public static String PROJECT_HOOK = "BnProjectHook";
	final public static String PROJECT_INTER_HOOK = "BnProjectInterHook";
	final public static String PROJECT_PROPERTY = "BnProjectProperty";
	final public static String ROLE = "BnRole";
	final public static String ROLE_MAPPER = "BnRoleMapper";
	final public static String USER = "BnUser";
	final public static String USER_PROPERTY = "BnUserProperty";
	final public static String ITERATION = "BnIteration";
	final public static String PROJECTCONFIG = "BnProjectConfig";

    static private  String getServiceName(int service) {
	  switch (service) {
	    case Services.TOPIC_CONNECTION_FACTORY: return TOPIC_CONNECTION_FACTORY_NAME;
		case Services.TOPIC: return TOPIC_NAME;
		case Services.MAIL_SERVICE: return MAIL_SERVICE_NAME;
	  }
	return null;
    }

    // returns a EJBHome for the given service, using the JNDI name
    // and the Class for the EJBHome

    public Object getResource(int s) throws BonitaServiceException {
	Object resource=null;
	try {
		String serv = getServiceName(s);
		if (cache.containsKey(serv)) {
		  resource = (Object) cache.get(serv);
		} else {
	       Object objref = initial.lookup(serv);
	       resource=objref;
		   cache.put(serv, resource);
		}
	} catch (NamingException ex) {
	    throw new BonitaServiceException("ServiceException");
	} catch (Exception ex) {
	    System.out.println("ServiceLocator Exception");
	    ex.printStackTrace();
	}

	return resource;
    }   

	public Object getMailSession(int s) throws BonitaServiceException {
	Object mail=null;
	try {
		String serv = getServiceName(s);
		if (cache.containsKey(serv)) {
		  mail = (Object) cache.get(serv);
		} else {
		   Object objref = initial.lookup(serv);
		   mail= PortableRemoteObject.narrow(objref,Session.class);
		   cache.put(serv, mail);
		}
	} catch (NamingException ex) {
		throw new BonitaServiceException("ServiceException");
	} catch (Exception ex) {
		System.out.println("ServiceLocator Exception");
		ex.printStackTrace();
	}

	return mail;
	}   

	/**
	 * Retrive a unique creation id to use for this bean.  This will end up
	 * demarcating this bean from others when it is stored as a record
	 * in the database.
	 *
	 * @return Returns an integer that can be used as a unique creation id.
	 *
	 * @throws ServiceUnavailableException Indicating that it was not possible
	 *                                     to retrieve a new unqiue ID because
	 *                                     the service is not available
	**/
	public int generateUniqueId(String sequenceName) throws BonitaServiceException
	{
	SequenceGeneratorHome resource=null;
	String sequence="sequence";
	 int lUniqueId = -1;
	 try {
		if (cache.containsKey(sequence)) {
		  resource = (SequenceGeneratorHome) cache.get(sequence);
		} else {
		  //Object obj = initial.lookup("java:comp/env/ejb/hero/SequenceGenerator");
		  //resource = (SequenceGeneratorHome) PortableRemoteObject.narrow(obj,SequenceGeneratorHome.class);
			resource = (SequenceGeneratorHome)hero.interfaces.SequenceGeneratorUtil.getHome();
		  cache.put(sequence, resource);
		}
		  SequenceGenerator lBean = (SequenceGenerator) resource.create();
		  lUniqueId = lBean.getNextNumber( sequenceName );
		  lBean.remove();
		  
	    }catch (ServiceUnavailableException sue){
		   throw new BonitaServiceException( "Service Unavailable Exception when calling getnextNumber: " + sue.getMessage() );
		}catch ( NamingException ne ) {
			throw new BonitaServiceException( "Naming lookup failure: " + ne.getMessage() );
	    }catch ( CreateException ce ) {
			throw new BonitaServiceException( "Failure while creating a generator session bean: " + ce.getMessage() );
		}catch ( RemoveException re ) {
			// When the Bean cannot be removed after a while it will be taken back by the container
			// therefore ignore this exception
		}catch ( RemoteException rte ) {
			throw new BonitaServiceException( "Remote exception occured while accessing generator session bean: " +  rte.getMessage() );
	    }
      
		 return lUniqueId;
	  }
    
}
