/*
 * Created on 3 mai 2004
 *
 */
package mc.formgenerator.bonita;

import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;

import java.util.*;

import org.w3c.dom.Document;

/**
 * @author delfourr
 */
public class BonitaActivityExecutor {



	/**
	 * Default constructor
	 */
	public BonitaActivityExecutor() {
		
	}
	
	
	
	/**
	 * Allows to find, initialise and terminate an activity.
	 * @param document Document with the activity information (name, attributes and project name).
	 * @param mode the application mode: consumer or exploitant
	 * @throws LoginException
	 */
	public void actOnActivity(Document document, String mode){
		
		//'parse' transforms a Document
		DocumentParser parse = new DocumentParser();

		DataActivity data = parse.parseActivity(document);
		
				
		//get the project name to wich it belongs	
		String projectName = data.getProjectName();


		//get the process name	
		String activityName = data.getProcessName();


		//get the process properties
		HashMap table = data.getProcessProperties();
		
		//terminate the activity thanks to its name and properties
		this.startActivity(activityName, projectName, table, mode);		
	}
	
	
	
	
	
	/**
	 * Thanks to its name, the activity is found and stopped. 
	 * Before, its attributes are set with the HashMap values.
	 * @param activityName Name of the activity to stop.
	 * @param projectName Name of the project to which it belongs.
	 * @param table HashMap with the activity attributes associated to their values
	 * @param mode the application mode: consumer or exploitant
	 * @throws LoginException
	 */
	private void startActivity(String activityName, String projectName, HashMap table, String mode) {

		//The local home
		ProjectSessionHome projecth = null;

		//The bean
		ProjectSession project = null;

		try {
			//Getting local home by using JNDI
			projecth = ProjectSessionUtil.getHome();

			//Creation of the project
			project = projecth.create();
				
			project.initProject(projectName);
				

			//**************************************************************************
			//					SET THE ACTIVITY PROPERTIES
			//**************************************************************************

			//All the keys are saved in a set
			Set collec = table.keySet();

			//Set properties if they exist
			if (collec.size() > 0) {

				//For all the keys we have to retrieve the associated value
				Iterator it = collec.iterator();

				//variable which will stock the property name
				String cle="";

				while (it.hasNext()) {

					//Getting the values
					cle = (String) it.next();

					//Set the current process property
					project.setNodeProperty(activityName, cle, (String) table.get(cle));
				}
			}
			
			//**************************************************************************
			//					START THE ACTIVITY
			//**************************************************************************
			
			//The local home
			UserSessionHome userh = UserSessionUtil.getHome();

			//The bean
			UserSession user = userh.create();		
			//the session starts the activity
			user.startActivity(projectName, activityName);
			
			//**************************************************************************
			//					STOP THE ACTIVITY
			//**************************************************************************	
			
			if(mode.equals("consumer"))
			    this.endActivity(activityName, projectName);
			
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}

	
	


	public void endActivity (String activityName, String projectName) {
		try {
			
			//The local home
			UserSessionHome userh = UserSessionUtil.getHome();
		
			//The bean
			UserSession user = userh.create();
			
			
			//the session terminates the activity
			user.terminateActivity( projectName, activityName);
			
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}
	
}
